#include "common.h"
#include "value_type_def.h"

#include <stdio.h>

#define construction_equal  "types in tuples should be equal"
#define construction_basic  "types in tuples should be basic"
#define construction_string "string tuples are forbidden"
#define construction_matrix "only float matrices are allowed"

#define make_couple_fun_name "make_couple_type"
#define make_triple_fun_name "make_triple_type"

#define construction_equal_error(fun_name) error (fun_name, construction_equal)
#define construction_basic_error(fun_name) error (fun_name, construction_basic)
#define construction_string_error(fun_name) error (fun_name, construction_string)
#define construction_matrix_error(fun_name) error (fun_name, construction_matrix)

vtd_scalar_type make_couple_type (vtd_scalar_type fst, vtd_scalar_type snd)
{
  if (fst != snd)
    construction_equal_error (make_couple_fun_name);
  else switch (fst) {
    case VTD_Bool_type:		return VTD_Couple_Bool_type;
    case VTD_Int_type:		return VTD_Couple_Int_type;
    case VTD_Native_Int_type:	return VTD_Couple_Native_Int_type;
    case VTD_Int32_type:	return VTD_Couple_Int32_type;
    case VTD_Int64_type:	return VTD_Couple_Int64_type;
    case VTD_Float_type:	return VTD_Couple_Float_type;
    case VTD_Float_bin_type:	return VTD_Couple_Float_bin_type;
    case VTD_String_type:	construction_string_error (make_couple_fun_name);
    default:
      construction_basic_error (make_couple_fun_name);
  }
}

vtd_scalar_type make_triple_type (vtd_scalar_type fst, vtd_scalar_type snd, vtd_scalar_type thrd) {
  if (fst != snd || fst != thrd)
    construction_equal_error (make_triple_fun_name);
  else switch (fst) {
    case VTD_Bool_type:		return VTD_Triple_Bool_type;
    case VTD_Int_type:		return VTD_Triple_Int_type;
    case VTD_Native_Int_type:	return VTD_Triple_Native_Int_type;
    case VTD_Int32_type:	return VTD_Triple_Int32_type;
    case VTD_Int64_type:	return VTD_Triple_Int64_type;
    case VTD_Float_type:	return VTD_Triple_Float_type;
    case VTD_Float_bin_type:	return VTD_Triple_Float_bin_type;
    case VTD_String_type:	construction_string_error (make_triple_fun_name);
    default:
      construction_basic_error (make_triple_fun_name);
  }
}

int vtd_check_value_type (vtd_value_type vt)
{
  vtd_value_type vtd_value_type_sans_scalar = vt & VTD_TYPE_MASK;

  switch (vtd_value_type_sans_scalar) {
  case VTD_SCALAR_TYPE:
  case VTD_VECTOR_TYPE:
  case VTD_MATRIX_TYPE:
    return vtd_value_type_sans_scalar;
  default:
    error ("vtd_check_value_type", "conversion error");
  }
}

vtd_value_type vtd_matrix_type (vtd_scalar_type typ)
{
  switch (typ) {
  case VTD_Float_bin_type:
  case VTD_Float_type:
    return (vtd_scalar_type) (((int) typ) | VTD_MATRIX_TYPE);
  default:
    construction_matrix_error ("vtd_matrix_type");
  }
}
