(***********************************************************************)
(*                                                                     *)
(*                          Sklml                                      *)
(*                                                                     *)
(*             Pierre Weis (Pierre.Weis@inria.fr)                      *)
(*             Francois Clement (Francois.Clement@inria.fr)            *)
(*             Quentin Carbonneaux (Quentin.Carbonneaux@inria.fr)      *)
(*                                                                     *)
(* This program is free software.                                      *)
(* (see the file LICENSE in the root directory of the distribution).   *)
(*                                                                     *)
(***********************************************************************)

(* $Id: IO.mli,v 1.1 2011-02-25 11:30:00 weis Exp $ *)

type ic;;
type oc;;

type pid = int;;
type fork_data =
   | Child
   | Parent of pid
;;

exception IOError of string;;

val stdin : ic;;
val stdout : oc;;

val ioerr : string -> 'a;;
val read_sockaddr : string -> Unix.sockaddr;;
val print_sockaddr : out_channel -> Unix.sockaddr -> unit;;
val get_port : Unix.sockaddr -> int;;
val create_bound_socket : int option -> Unix.file_descr * Unix.sockaddr;;
val connect : Unix.sockaddr -> ic * oc;;
val close_in : ic -> unit;;
val close_out : oc -> unit;;
(* val flush : oc -> unit;; *)
val connect_fd : Unix.sockaddr -> Unix.file_descr;;
val of_channel : ic -> 'a;;
val to_channel : oc -> 'a -> bool -> unit;;
val in_channel_of_ic : ic -> in_channel;;
val out_channel_of_oc : oc -> out_channel;;
val wait_connection : Unix.file_descr -> bool ->
  (ic * oc) * Unix.inet_addr;;
val fork : unit -> fork_data;;
val open_process : string -> (ic * oc);;
val open_process_in : string -> ic;;
